/* -----------------------------------------------------------------------------
 - File:      nRF24l01 Receive Test
 - Copyright: Copyright (C) 2015 SunFounder. For details, check License folder.
 - Date:      2015/7/21
 * -----------------------------------------------------------------------------
 - Overview
  - This project was written to test nRF24l01 module. If it runs, arduino will 
    reads Analog values on A5, A6 and transmits them over a nRF24L01 Radio Link 
    to another transceiver.
 - Request
   - FR24 library
 - Connections
   - nRF24L01 to Arduino
     1 GND   	 GND
     2 VCC	 3V3
     3 CE	 D9
     4 CSN	 D10
     5 SCK	 D13
     6 MOSI	 D11
     7 MISO	 D12
     8 UNUSED	
   - Joystick to Arduino
     GND         GND
     VCC         5V
     X           A5
     Y           A6
	 
 * ---------------------------------------------------------------------------*/

/* Includes ------------------------------------------------------------------*/
#include <nRF24L01.h>
#include <RF24_config.h>
#include <RF24.h>

/* Ports ---------------------------------------------------------------------*/
#define CE_PIN   9
#define CSN_PIN 10

/* nRF24l01 ------------------------------------------------------------------*/
// NOTA: le "LL" alla fine della costante definisce il tipo "LongLong"
// Indirizzo di default
const uint64_t address = 0xE7E7E7E7E7LL;
// Creo una Radio 
RF24 radio(CE_PIN, CSN_PIN); 

/* Joystick ------------------------------------------------------------------*/
// array con 2 elementi contenenti le letture del joystick
int joystick[2]; 
/* ---------------------------------------------------------------------------*/

/*
 - setup 
 * ---------------------------------------------------------------------------*/
void setup()
{
  // inizializzo la porta seriale
  Serial.begin(9600);
  delay(1000);
  Serial.println("nRF24l01 Receiver Starting");
  
  // inizializzo la radio... questa istruzione va sempre scritta per prima!!!
  radio.begin();
  
  // abilito ed imposto il CRC ad 8 bit
  radio.setCRCLength(RF24_CRC_8);
  
  // abilito l'auto ACK
  radio.setAutoAck(1);
  
  // abilito l'ack payload (lavora solo sui pipes 0 e 1 per default)
  // per usarlo su tutte le pipes si deve abilitare il payload dinamico
  radio.enableAckPayload();
  
  // abilito il payload dinamico
  radio.enableDynamicPayloads();
  
  // imposto il canale di trasmissione (il canale 2 è quello di default)
  radio.setChannel(2);
  
  // imposto il ritardo ed il numero di tentativi necessari per ritrasmettere
  // il dato in caso di fallimento... 0 = 250 uS, 15 = max numero di tentativi
  radio.setRetries(0, 15);
  
  // imposto la potenza di trasmissione a -6dBM
  radio.setPALevel(RF24_PA_HIGH);
  
  // imposto la larghezza dell'indirizzo a 5 byte
  radio.setAddressWidth (0x05);
  
  // imposto l'indirizzo di lettura sul pipe 0
  radio.openReadingPipe(0,address);
  
  // imposto la radio in ricezione
  radio.startListening();;
}

/*
 - loop 
 * ---------------------------------------------------------------------------*/
void loop()
{
  // se c'è comunicazione tra i moduli
  if (radio.available())
  {
    // leggo tutto il payload (pacchetto dati) fino a quando termina
    bool done = false;
    while (!done)
    {
      radio.read( joystick, sizeof(joystick) );
	  
	  // invio i dati ricevuti sulla seriale
      Serial.print("X = ");
      Serial.print(joystick[0]);
      Serial.print(" Y = ");      
      Serial.println(joystick[1]);
    }
  }
  else
  {    
    // se non cìè comunicazione tra i moduli stampo il messaggio di errore
	Serial.println("No radio available");
  }
}
