/* -----------------------------------------------------------------------------
 - File:      nRF24l01 Transmit Test
 - Copyright: Copyright (C) 2015 SunFounder. For details, check License folder.
 - Date:      2015/7/21
 * -----------------------------------------------------------------------------
 - Overview
  - This project was written to test nRF24l01 module. If it runs, arduino will 
    reads Analog values on A5, A6 and transmits them over a nRF24L01 Radio Link 
    to another transceiver.
 - Request
   - FR24 library
 - Connections
   - nRF24L01 to Arduino
     1 GND   	 GND
     2 VCC	 3V3
     3 CE	 D9
     4 CSN	 D10
     5 SCK	 D13
     6 MOSI	 D11
     7 MISO	 D12
     8 UNUSED	
   - Joystick to Arduino
     GND         GND
     VCC         5V
     X           A5
     Y           A6

 * ---------------------------------------------------------------------------*/

/* Includes ------------------------------------------------------------------*/
#include <nRF24L01.h>
#include <RF24_config.h>
#include <RF24.h>

/* Ports ---------------------------------------------------------------------*/
#define CE_PIN   9
#define CSN_PIN 10
#define JOYSTICK_X A5
#define JOYSTICK_Y A6

/* nRF24l01 ------------------------------------------------------------------*/
// NOTA: le "LL" alla fine della costante definisce il tipo "LongLong"
// Indirizzo di default
const uint64_t address = 0xE7E7E7E7E7LL;
// Creo una Radio 
RF24 radio(CE_PIN, CSN_PIN); 

/* Joystick ------------------------------------------------------------------*/
// array con 2 elementi contenenti le letture del joystick
int joystick[2]; 
/* ---------------------------------------------------------------------------*/

/*
 - setup 
 * ---------------------------------------------------------------------------*/
void setup()
{
  // imposto i pin analogici in ingresso
  pinMode(JOYSTICK_X, INPUT);
  pinMode(JOYSTICK_Y, INPUT);
  
  // inizializzo la radio... questa istruzione va sempre scritta per prima!!!
  radio.begin();
  
  // abilito ed imposto il CRC ad 8 bit
  radio.setCRCLength(RF24_CRC_8);
  
  // abilito l'auto ACK
  radio.setAutoAck(1);
  
  // abilito l'ack payload (lavora solo sui pipes 0 e 1 per default)
  // per usarlo su tutte le pipes si deve abilitare il payload dinamico
  radio.enableAckPayload();
  
  // abilito il payload dinamico
  radio.enableDynamicPayloads();
  
  // imposto il canale di trasmissione (il canale 2 è quello di default)
  radio.setChannel(2);
  
  // imposto il ritardo ed il numero di tentativi necessari per ritrasmettere
  // il dato in caso di fallimento... 0 = 250 uS, 15 = max numero di tentativi
  radio.setRetries(0, 15);
  
  // imposto la potenza di trasmissione a -6dBM
  radio.setPALevel(RF24_PA_HIGH);
  
  // imposto la larghezza dell'indirizzo a 5 byte
  radio.setAddressWidth (0x05);
  
  // imposto l'indirizzo di scrittura
  radio.openWritingPipe(address);

  radio.stopListening();
}

/*
 - loop 
 * ---------------------------------------------------------------------------*/
void loop()
{
  // memorizzo i valori adc nelle due celle dell'array
  joystick[0] = analogRead(JOYSTICK_X);
  joystick[1] = analogRead(JOYSTICK_Y);
  
  // trasmetto i dati
  radio.write(joystick, sizeof(joystick));
}
